module MessageExchange
  class Request
    attr_reader :uri, :params

    def initialize(uri, params = {})
      @uri    = URI(uri)
      @params = params
    end

    def request
      raise NotImplementedError, "Subclasses must implement this"
    end

    def port
      80
    end

    def response
      @response ||= Response.new(http.request(request))
    rescue Timeout::Error
      @response ||= ServerErrorResponse.new
    end

    def http
      Net::HTTP.new(uri.host, port).tap do |http|
        http.open_timeout = 0.1
        http.read_timeout = 0.25
      end
    end
  end
end
